'Driver for Rayman's mono LCD (120x32) graphical
' Notes:
'  Text is 0..19 x 0..3, Plotting is (0..119) x (0..31)
'
'  There are 4 static icons on the top left (Light bulb, Antenna, Speaker, Speaker slash). These are controlled
'    by using the SetIcons commands with a value from 0 to 15.
'
'  The rest of the icons on the top line are controlled by plotting or unplotting on line 32.
'    Only these positions are used: 0,16,17..23,25..31,33..39,41..47,52, 60,68,73..79,81..87,88,89..95,97..103,107,115,119  
'
'  Routines:
'    ClearScreen     - Clears screen. Does NOT clear icons
'    ClearIcons      - Clear static and non-static icons
'    SetIcons(value) - Controls the 4 static icons.value=0..15
'    Pos(x, y)       - Sets text position. x=0..19, y=0..3
'    PutChar(char)   - Displays 1 character.
'    PutStr(string)  - Displays a string of characters.  
'    Plot(x, y)      - Plots pixel (or icon). x=0..119, y=0..32 (32 is for icons)
'    UnPlot(x, y)    - Unplots pixel (or icon). x=0..119, y=0..32 (32 is for icons)
'
'  Note: The SendCommand, and SendData routines will NOT work after the driver is started.   
'
'  Note: The driver continuously updates the LCD with the Screen[] data. All you need to do is change the screen[]
'        data, and the LCD will be updated by the driver automatically. 
'
VAR
  BYTE Icons
  BYTE Screen[600]
  BYTE ClockPin
  BYTE DCPin
  BYTE DataPins

  BYTE TextX, TextY  

  
PUB Init(_RstPin, _DCPin, _CSnPin, _DataPin) | i, x, y
  ClockPin := _CSnPin
  DCPin := _DCPin
  DataPins := _DataPin

  'set up pins
  OUTA[_RstPin]~~
  DIRA[_RstPin]~~

  OUTA[ClockPin]~
  DIRA[ClockPin]~~

  OUTA[DCPin]~~
  DIRA[DCPin]~~

  OUTA[DataPins..(DataPins+7)]~  
  DIRA[DataPins..(DataPins+7)]~~

  'Reset
  OUTA[_RstPin]~
  waitcnt(cnt+clkfreq/100)
  OUTA[_RstPin]~~
  waitcnt(cnt+clkfreq/100)

  'Initialize Display
  'Set Clock Frequency
  
  'Set Oscillator Enable
  SendCommand(%01111111)
  'Set Static Icon Control Signals
  'Set Duty Cycle
  'Set Bias Ratio
  'Set Internal DC/DC Converter On
  SendCommand(%00101011)
  'Set Internal Regulator On
  SendCommand(%00101101) '???
  'Set Temperature Coefficient
  'Set Internal Contrast Control On
  'SendCommand(%00110001)
  'Increase Contrast Level
  SendCommand(%00011000)
  'Set Internal Voltage Divider On
  SendCommand(%00101111)
  'Set Segment Mapping
  'Set Common Mapping
  'Set Vertical Scroll Value
  'Set Display On
  SendCommand(%00101001)


  'Turn off 4 main Icons
  SendCommand(%01100_000)
  SendCommand(%01100_010)
  SendCommand(%01100_100)
  SendCommand(%01100_110)


  'Enable on Extra Icons
  SendCommand(%00111101)

  'Clear GRAM
  ClearScreen
  ClearIcons

  'Change mapping so that ICONS are on top with text from left to right...
  SendCommand(%00100011)


  ' Make DC,Clock, and Data pins inputs so the PASM driver can use them
  dira[DCPin]~
  dira[ClockPin]~
  dira[DataPins..(DataPins+7)]~

  ' Start Driver
  CogNew(@LCDDriver, @Icons)


PUB Pos(x, y)
  TextX := x
  TextY := y

  
PUB PutChar(char) | idx
  if char == 13
    TextX := 0
    TextY += 1
    TextY <#= 3 ' Limit Y to maximum of 3
  else
    char := (char - 32) * 6 ' 6 Bytes per character
    idx := TextY * 120 + TextX * 6
    Screen[idx] := BYTE[@Font8x6+char]
    Screen[idx+1] := BYTE[@Font8x6+char+1]
    Screen[idx+2] := BYTE[@Font8x6+char+2]
    Screen[idx+3] := BYTE[@Font8x6+char+3]
    Screen[idx+4] := BYTE[@Font8x6+char+4]
    Screen[idx+5] := BYTE[@Font8x6+char+5]
    TextX += 1
    If TextX == 20
      TextX := 0
      TextY += 1
      TextY <#= 3 ' Limit Y to maximum of 3
     

PUB PutStr(ptr)
  repeat
    if byte[ptr] == 0
      quit
    PutChar(byte[ptr])
    ptr += 1  


PUB Plot(x, y)
  Screen[(y / 8) * 120 + x] |= 1 << (y // 8)


PUB UnPlot(x, y)
  Screen[(y / 8) * 120 + x]  &= !(1 << (y // 8))


PUB ClearScreen
  ByteFill(@Screen, 0, 480)
  TextX := 0
  TextY := 0


PUB ClearIcons                         
  ByteFill(@Screen[480], 0, 120)
  Icons := 0

  
PUB SetIcons(value)
  Icons := value
    

PUB SendCommand(cmd)
  'Send a command
  OUTA[DataPins+7..DataPins]:=cmd
  OUTA[DCPin]~
  OUTA[ClockPin]~~  'cycle clock
  OUTA[ClockPin]~
  
  
PUB SendData(cmd)
  'Send a command
  OUTA[DataPins+7..DataPins]:=cmd
  OUTA[DCPin]~~
  OUTA[ClockPin]~~  'cycle clock
  OUTA[ClockPin]~
  
  
DAT
  ORG 0
  ' Continuously send data to LCD display

  ' Setup pins
LCDDriver
  mov _addr,par
  add _addr,#300 ' Move past screen data
  add _addr,#301 ' Move past screen data & Icon data

  rdbyte _temp,_addr ' Get Clock pin number
  shl _ClockPinMask,_temp
  or dira,_ClockPinMask ' Make Clock pin an output
  andn outa,_ClockPinMask ' Make Clock pin low

  add _addr,#1
  rdbyte _temp,_addr ' Get Data_nCmd pin number
  shl _Data_nCmdPinMask,_temp
  or dira,_Data_nCmdPinMask ' Make Data_nCmd pin an output
  andn outa,_Data_nCmdPinMask ' Make Data_nCmd pin low

  add _addr,#1
  rdbyte _DataPinsShift,_addr ' Get Data Pins number
  shl _DataPinsMask,_DataPinsShift
  or dira,_DataPinsMask ' Make Data pins outputs

  rdlong _ClockCnt,#0 ' Get frequency
  shr _ClockCnt,#19   ' 1.9uSec
  min _ClockCnt,#20   ' Not less than 20 cycles  

Again
  mov _addr,par
  mov _page,#5 ' Send 5 pages
  
 ' Setup to send commands
  andn outa,_Data_nCmdPinMask
  mov _Wait,_ClockCnt
  add _Wait,cnt
  waitcnt _Wait,_ClockCnt
  
  ' Send Static Icon control commands
  rdbyte _temp,_addr
  and _temp,#1
  add _temp,#%01100_000
  shl _temp,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_temp ' Set pins that are "1"
  call #ToggleClk  

  rdbyte _temp,_addr
  and _temp,#2
  shr _temp,#1
  add _temp,#%01100_010
  shl _temp,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_temp ' Set pins that are "1"
  call #ToggleClk

  rdbyte _temp,_addr
  and _temp,#4
  shr _temp,#2
  add _temp,#%01100_100
  shl _temp,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_temp ' Set pins that are "1"
  call #ToggleClk

  rdbyte _temp,_addr
  and _temp,#8
  shr _temp,#3  
  add _temp,#%01100_110
  shl _temp,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_temp ' Set pins that are "1"
  call #ToggleClk

  add _addr,#1 

NextPage
  mov _colCnt,#120

 ' Setup to send commands
  andn outa,_Data_nCmdPinMask
  mov _Wait,_ClockCnt
  add _Wait,cnt
  waitcnt _Wait,_ClockCnt

  ' Select page 
  mov _temp,#5
  sub _temp,_page
  shl _temp,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_temp ' Set pins that are "1"
  call #ToggleClk

  ' Select column 0
  mov _temp,#128
  shl _temp,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_temp ' Set pins that are "1"
  call #ToggleClk

  or outa,_Data_nCmdPinMask ' Setup to send data now    
  waitcnt _Wait,_ClockCnt

NextCol
  rdbyte _col, _addr
  add _addr,#1

  ' Put data on D0..D7 pins
  shl _col,_DataPinsShift
  andn outa,_DataPinsMask ' Make data pins all low
  or outa,_col ' Set pins that are "1"
  call #ToggleClk  

  djnz _colCnt,#NextCol

  djnz _page,#NextPage

  jmp #Again


ToggleClk
  ' Toggle clock
  or outa,_ClockPinMask
  mov _Wait,_ClockCnt
  add _Wait,cnt
  waitcnt _Wait,_ClockCnt
  andn outa,_ClockPinMask
  waitcnt _Wait,_ClockCnt
ToggleClk_ret
  ret

_ClockPinMask       LONG 1
_Data_nCmdPinMask   LONG 1
_DataPinsMask       LONG 255

_page               RES 1
_addr               RES 1
_temp               RES 1
_col                RES 1
_colCnt             RES 1
_DataPinsShift      RES 1
_Wait               RES 1
_ClockCnt           RES 1


Font8x6
        BYTE     $00,$00,$00,$00,$00,$00 ' space (32)
        BYTE     $00,$00,$5F,$00,$00,$00 
        BYTE     $00,$07,$00,$07,$00,$00
        BYTE     $14,$7F,$14,$7F,$14,$00
        BYTE     $24,$2A,$7F,$2A,$12,$00
        BYTE     $23,$13,$08,$64,$62,$00
        BYTE     $36,$49,$56,$20,$50,$00
        BYTE     $00,$08,$07,$03,$00,$00
        BYTE     $00,$1C,$22,$41,$00,$00
        BYTE     $00,$41,$22,$1C,$00,$00
        BYTE     $2A,$1C,$7F,$1C,$2A,$00
        BYTE     $08,$08,$3E,$08,$08,$00
        BYTE     $00,$80,$70,$30,$00,$00
        BYTE     $08,$08,$08,$08,$08,$00
        BYTE     $00,$00,$60,$60,$00,$00
        BYTE     $20,$10,$08,$04,$02,$00
        BYTE     $3E,$51,$49,$45,$3E,$00
        BYTE     $00,$42,$7F,$40,$00,$00
        BYTE     $72,$49,$49,$49,$46,$00
        BYTE     $21,$41,$49,$4D,$33,$00
        BYTE     $18,$14,$12,$7F,$10,$00
        BYTE     $27,$45,$45,$45,$39,$00
        BYTE     $3C,$4A,$49,$49,$31,$00
        BYTE     $41,$21,$11,$09,$07,$00
        BYTE     $36,$49,$49,$49,$36,$00
        BYTE     $46,$49,$49,$29,$1E,$00
        BYTE     $00,$00,$14,$00,$00,$00
        BYTE     $00,$40,$34,$00,$00,$00
        BYTE     $00,$08,$14,$22,$41,$00
        BYTE     $14,$14,$14,$14,$14,$00
        BYTE     $00,$41,$22,$14,$08,$00
        BYTE     $02,$01,$59,$09,$06,$00
        BYTE     $3E,$41,$5D,$59,$4E,$00
        BYTE     $7C,$12,$11,$12,$7C,$00 ' "A" (65)
        BYTE     $7F,$49,$49,$49,$36,$00
        BYTE     $3E,$41,$41,$41,$22,$00
        BYTE     $7F,$41,$41,$41,$3E,$00
        BYTE     $7F,$49,$49,$49,$41,$00
        BYTE     $7F,$09,$09,$09,$01,$00
        BYTE     $3E,$41,$41,$51,$73,$00
        BYTE     $7F,$08,$08,$08,$7F,$00
        BYTE     $00,$41,$7F,$41,$00,$00
        BYTE     $20,$40,$41,$3F,$01,$00
        BYTE     $7F,$08,$14,$22,$41,$00
        BYTE     $7F,$40,$40,$40,$40,$00
        BYTE     $7F,$02,$1C,$02,$7F,$00
        BYTE     $7F,$04,$08,$10,$7F,$00
        BYTE     $3E,$41,$41,$41,$3E,$00
        BYTE     $7F,$09,$09,$09,$06,$00
        BYTE     $3E,$41,$51,$21,$5E,$00
        BYTE     $7F,$09,$19,$29,$46,$00
        BYTE     $26,$49,$49,$49,$32,$00
        BYTE     $03,$01,$7F,$01,$03,$00
        BYTE     $3F,$40,$40,$40,$3F,$00
        BYTE     $1F,$20,$40,$20,$1F,$00
        BYTE     $3F,$40,$38,$40,$3F,$00
        BYTE     $63,$14,$08,$14,$63,$00
        BYTE     $03,$04,$78,$04,$03,$00
        BYTE     $61,$59,$49,$4D,$43,$00 ' "Z" (90)
        BYTE     $00,$7F,$41,$41,$41,$00
        BYTE     $02,$04,$08,$10,$20,$00
        BYTE     $00,$41,$41,$41,$7F,$00
        BYTE     $04,$02,$01,$02,$04,$00   
        BYTE     $40,$40,$40,$40,$40,$00
        BYTE     $00,$03,$07,$08,$00,$00
        BYTE     $20,$54,$54,$78,$40,$00 ' "a" (97)
        BYTE     $7F,$28,$44,$44,$38,$00
        BYTE     $38,$44,$44,$44,$28,$00
        BYTE     $38,$44,$44,$28,$7F,$00
        BYTE     $38,$54,$54,$54,$18,$00
        BYTE     $00,$08,$7E,$09,$02,$00
        BYTE     $18,$A4,$A4,$9C,$78,$00
        BYTE     $7F,$08,$04,$04,$78,$00
        BYTE     $00,$44,$7D,$40,$00,$00
        BYTE     $20,$40,$40,$3D,$00,$00
        BYTE     $7F,$10,$28,$44,$00,$00
        BYTE     $00,$41,$7F,$40,$00,$00
        BYTE     $7C,$04,$78,$04,$78,$00
        BYTE     $7C,$08,$04,$04,$78,$00
        BYTE     $38,$44,$44,$44,$38,$00
        BYTE     $FC,$18,$24,$24,$18,$00
        BYTE     $18,$24,$24,$18,$FC,$00
        BYTE     $7C,$08,$04,$04,$08,$00
        BYTE     $48,$54,$54,$54,$24,$00
        BYTE     $04,$04,$3F,$44,$24,$00
        BYTE     $3C,$40,$40,$20,$7C,$00
        BYTE     $1C,$20,$40,$20,$1C,$00
        BYTE     $3C,$40,$30,$40,$3C,$00
        BYTE     $44,$28,$10,$28,$44,$00
        BYTE     $4C,$90,$90,$90,$7C,$00
        BYTE     $44,$64,$54,$4C,$44,$00  ' "z" (122)
        BYTE     $00,$08,$36,$41,$00,$00
        BYTE     $00,$00,$77,$00,$00,$00
        BYTE     $00,$41,$36,$08,$00,$00
        BYTE     $02,$01,$02,$04,$02,$00 '
        BYTE     $3C,$26,$23,$26,$3C,$00 ' (127)

  